/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is The Browser Profile Migrator.
 *
 * The Initial Developer of the Original Code is Ben Goodger.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Ben Goodger <ben@bengoodger.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#ifndef firefoxprofilemigrator___h___
#define firefoxprofilemigrator___h___

#include "nsIBrowserProfileMigrator.h"
#include "nsIObserverService.h"
#include "nsNetscapeProfileMigratorBase.h"
#include "nsILineInputStream.h"
#include "nsINavHistoryService.h"


class nsIPrefBranch;
class nsIPrefService;
class nsVoidArray;

class nsFirefoxProfileMigrator : public nsNetscapeProfileMigratorBase, 
                                 public nsIBrowserProfileMigrator,
                                 public nsINavHistoryBatchCallback
{
public:
  NS_DECL_NSIBROWSERPROFILEMIGRATOR
  NS_DECL_NSINAVHISTORYBATCHCALLBACK
  NS_DECL_ISUPPORTS

  nsFirefoxProfileMigrator();
  virtual ~nsFirefoxProfileMigrator();

public:
  static nsresult SetImage(void* aTransform, nsIPrefBranch* aBranch);
  static nsresult SetCookie(void* aTransform, nsIPrefBranch* aBranch);
  static nsresult SetDownloadManager(void* aTransform, nsIPrefBranch* aBranch);

protected:
  nsresult FillProfileDataFromRegistry();
  nsresult GetSourceProfile(const PRUnichar* aProfile);

  nsresult CopyPreferences(PRBool aReplace);
  nsresult TransformPreferences(const nsAString& aSourcePrefFileName,
                                const nsAString& aTargetPrefFileName);
  void     ReadFontsBranch(nsIPrefService* aPrefService, nsVoidArray* aPrefs);
  void     WriteFontsBranch(nsIPrefService* aPrefService, nsVoidArray* aPrefs);

  nsresult CopyUserStyleSheets();

  nsresult CopyCookies(PRBool aReplace);
  nsresult CopyHistory(PRBool aReplace);
  nsresult CopyBookmarks(PRBool aReplace);
  nsresult CopyFormHistory(PRBool aReplace);
  nsresult CopyOtherData(PRBool aReplace);

  PRBool   SourceFileExists(const nsAString& aSourceFileName);

  void     MigrateYahooToolbar(void);

protected:
  virtual nsresult AppendNativeProfilePath(nsILocalFile *aFile);
  virtual nsresult GetBookmarksSourceName(nsAString &aName);
  virtual nsresult GetAppFileName(nsAString &aFileName);
  virtual nsresult CopyPasswords(PRBool aReplace);
  virtual PRBool   SetSourceSignonFileName(PRBool aReplace);

  nsString mSourceSignonFileName;
  nsString mSourceNameKey;  // For use by bookmarks import callback.

private:
  nsCOMPtr<nsISupportsArray> mProfileNames;
  nsCOMPtr<nsISupportsArray> mProfileLocations;
  nsCOMPtr<nsIObserverService> mObserverService;
};

// Netscape is effectively Firefox for our purposes, except
// the profile directory and application name are different
class nsNetscape9ProfileMigrator : public nsFirefoxProfileMigrator
{
protected:
  nsresult AppendNativeProfilePath(nsILocalFile *aFile);
  nsresult GetBookmarksSourceName(nsAString &aName);
  nsresult GetAppFileName(nsAString &aFileName);
};

#if defined(XP_WIN)

// Netscape 8 also has different password migration logic
class nsNetscape8ProfileMigrator : public nsFirefoxProfileMigrator
{
protected:
  nsresult AppendNativeProfilePath(nsILocalFile *aFile);
  nsresult GetBookmarksSourceName(nsAString &aName);
  nsresult GetAppFileName(nsAString &aFileName);
  nsresult CopyPasswords(PRBool aReplace);
  PRBool   SetSourceSignonFileName(PRBool aReplace);

private:
  nsresult ReadObfuscatedLine(nsILineInputStream* is,
                              nsAString& buffer,
                              PRBool* moreData);
};

#endif
 
#endif
