/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Places.
 * (derived from head_bookmarks.js, Places unit testing)
 *
 * The Initial Developer of the Original Code is
 * Google Inc.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Brian Ryner <bryner@brianryner.com>
 *  Dietrich Ayala <dietrich@mozilla.com>
 *  Flock Inc. <contact@flock.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


const NS_APP_USER_PROFILE_50_DIR = "ProfD";
var CI = Components.interfaces;
var CC = Components.classes;
var CR = Components.results;

function LOG(aMsg) {
  aMsg = ("*** FLOCK COMMON TESTS: " + aMsg);
  CC["@mozilla.org/consoleservice;1"]
  .getService(CI.nsIConsoleService)
  .logStringMessage(aMsg);
  print(aMsg);
}

// If there's no location registered for the profile directory, register one now.
var dirSvc = CC["@mozilla.org/file/directory_service;1"]
             .getService(CI.nsIProperties);
var profileDir = null;
try {
  profileDir = dirSvc.get(NS_APP_USER_PROFILE_50_DIR, CI.nsIFile);
} catch (e) {}
if (!profileDir) {
  // Register our own provider for the profile directory.
  // It will simply return the current directory.
  var provider = {
    getFile: function(prop, persistent) {
      persistent.value = true;
      if (prop == NS_APP_USER_PROFILE_50_DIR) {
        LOG(dirSvc.get("CurProcD", CI.nsIFile).spec);
        return dirSvc.get("CurProcD", CI.nsIFile);
      }
      throw CR.NS_ERROR_FAILURE;
    },
    QueryInterface: function(iid) {
      if (iid.equals(CI.nsIDirectoryServiceProvider) ||
          iid.equals(CI.nsISupports)) {
        return this;
      }
      throw CR.NS_ERROR_NO_INTERFACE;
    }
  };
  dirSvc.QueryInterface(CI.nsIDirectoryService).registerProvider(provider);
}

var XULAppInfo = {
  vendor: "Flock",
  name: "CoopTest",
  ID: "{63a2adaa-70ed-4261-a875-dc4a84d7093b}",
  version: "1",
  appBuildID: "2008010101",
  platformVersion: "",
  platformBuildID: "2008010101",
  inSafeMode: false,
  logConsoleErrors: true,
  OS: "XPCShell",
  XPCOMABI: "noarch-spidermonkey",

  QueryInterface: function QueryInterface(iid) {
    if (iid.equals(CI.nsIXULAppInfo) ||
        iid.equals(CI.nsIXULRuntime) ||
        iid.equals(CI.nsISupports))
      return this;
    throw CR.NS_ERROR_NO_INTERFACE;
  }
};

var XULAppInfoFactory = {
  createInstance: function (outer, iid) {
    if (outer != null)
      throw Cr.NS_ERROR_NO_AGGREGATION;
    return XULAppInfo.QueryInterface(iid);
  }
};

var registrar = Components.manager.QueryInterface(CI.nsIComponentRegistrar);
registrar.registerFactory(Components.ID("{fbfae60b-64a4-44ef-a911-08ceb70b9f31}"),
                          "XULAppInfo", "@mozilla.org/xre/app-info;1",
                          XULAppInfoFactory);
var updateSvc = CC["@mozilla.org/updates/update-service;1"]
                .getService(CI.nsISupports);
var iosvc = CC["@mozilla.org/network/io-service;1"]
            .getService(CI.nsIIOService);

function uri(spec) {
  return iosvc.newURI(spec, null, null);
}

// Make sure Coop is present
var flockRDF = Components.classes["@mozilla.org/rdf/rdf-service;1"]
                         .getService(Components.interfaces.nsIRDFService)
                         .GetDataSource('rdf:flock-favorites');
var obs = CC["@mozilla.org/observer-service;1"]
          .getService(CI.nsIObserverService);
obs.notifyObservers(null, "profile-after-change", false);
var coop = CC["@flock.com/singleton;1"]
           .getService(CI.flockISingleton)
           .getSingleton("chrome://flock/content/common/load-faves-coop.js")
           .wrappedJSObject;

function cleanUp() {
  try {
    // Delete a previously created flock-data file
    var file = dirSvc.get('ProfD', CI.nsIFile);
    file.append("flock-data.sqlite");
    if (file.exists()) {
      file.remove(false);
    }
  } catch(ex) {
    dump("Exception: " + ex);
  }
}
cleanUp();
