// BEGIN FLOCK GPL
//
// Copyright Flock Inc. 2005-2009
// http://flock.com
//
// This file may be used under the terms of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// END FLOCK GPL

const CC = Components.classes;
const CI = Components.interfaces;
const CR = Components.results;
const CU = Components.utils;

CU.import("resource:///modules/FlockSvcUtils.jsm");
CU.import("resource:///modules/FlockPrefsUtils.jsm");

// Service Enabled Pref
const SERVICE_ENABLED_PREF = "flock.service.onet.enabled";
const ONET_API_CONTRACT_ID = "@flock.com/webservice/api/onet;1";

var EXPORTED_SYMBOLS = ["OnetUtils"];

var OnetUtils = {
  api: CC[ONET_API_CONTRACT_ID].createInstance(CI.flockIOnetAPI),
  /**
   * Initialize an Onet service including the web detective file
   */
  initService: function Utils_initService(aService, aClassName, aContractId) {
    FlockSvcUtils.getLogger(aService);
    aService._logger.init(aClassName);
    aService._profiler = CC["@flock.com/profiler;1"]
                         .getService(CI.flockIProfiler);
    var evtID = aService._profiler.profileEventStart("onet-init");

    if (FlockPrefsUtils.getBoolPref(SERVICE_ENABLED_PREF) === false) {
      aService._logger.info("Pref "+ SERVICE_ENABLED_PREF
                            + " set to FALSE... not initializing.");
      aService.deleteCategories();
      return;
    }

    // Convenience variable
    aService._wd = FlockSvcUtils.getWD(aService);
    var authenticatedUrn = FlockSvcUtils.initService(aService, true);
    if (authenticatedUrn) {
      var c_acct = aService._coop.Account.get(authenticatedUrn);
      var svcUrn = "urn:" + aClassName + ":service:" + c_acct.accountId;
      this.api.authenticate(aService._acUtils.getPassword(svcUrn), null);
    }

    // Load auth manager
    aService._authMgr = CC["@flock.com/authmgr;1"]
                        .getService(CI.flockIAuthenticationManager);
    aService._authMgr.joinNetwork(aContractId, "onet");

    // Add default flockIWebService methods
    var ws = FlockSvcUtils.flockIWebService;
    ws.addDefaultMethod(aService, "url");
    ws.addDefaultMethod(aService, "getStringBundle");
    ws.addDefaultMethod(aService, "isHidden");

    // Add default flockILoginWebService methods
    var lws = FlockSvcUtils.flockILoginWebService;
    lws.addDefaultMethod(aService, "getAccount");
    lws.addDefaultMethod(aService, "getAccountIDFromDocument");
    lws.addDefaultMethod(aService, "getAuthenticatedAccount");
    lws.addDefaultMethod(aService, "getAccountUrns");
    lws.addDefaultMethod(aService, "getSessionValue");
    lws.addDefaultMethod(aService, "removeAccount");
    lws.addDefaultMethod(aService, "docRepresentsSuccessfulLogin");
    lws.addDefaultMethod(aService, "getCredentialsFromForm");
    lws.addDefaultMethod(aService, "ownsDocument");
    lws.addDefaultMethod(aService, "ownsLoginForm");
    lws.addDefaultMethod(aService, "tooltip");

    aService._profiler.profileEventEnd(evtID, "");
  },

  /**
   * Login the account and authenticate api
   */
  loginAccount: function Utils_loginAccount(aAcct, aClassName, aFlockListener) {
    aAcct._logger.debug("Utils_loginAccount()");

    if (!this.api.authenticated) {
      aAcct._acUtils.ensureOnlyAuthenticatedAccount(aAcct);
      var svcUrn = "urn:" + aClassName + ":service:" + aAcct.coopObj.accountId;
      this.api.authenticate(aAcct._acUtils.getPassword(svcUrn), aFlockListener);
    }
  }
};
