# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is the Mozilla Installer code.
#
# The Initial Developer of the Original Code is Mozilla Foundation
# Portions created by the Initial Developer are Copyright (C) 2006
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#  Robert Strong <robert.bugzilla@gmail.com>
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

# Required Plugins:
# AppAssocReg http://nsis.sourceforge.net/Application_Association_Registration_plug-in
# ShellLink   http://nsis.sourceforge.net/ShellLink_plug-in
# UAC         http://nsis.sourceforge.net/UAC_plug-in

; Set verbosity to 3 (e.g. no script) to lessen the noise in the build logs
!verbose 3

; 7-Zip provides better compression than the lzma from NSIS so we add the files
; uncompressed and use 7-Zip to create a SFX archive of it
SetDatablockOptimize on
SetCompress off
CRCCheck on

!addplugindir ./

; empty files - except for the comment line - for generating custom pages.
!system 'echo ; > welcome.ini'
!system 'echo ; > shortcuts.ini'
!system 'echo ; > finish.ini'

; USE_UAC_PLUGIN is temporary until all applications have been updated to use
; the UAC plugin
!define USE_UAC_PLUGIN

Var HWND
Var EXPRESSMODE
Var TmpVal
Var StartMenuDir
Var InstallType
Var AddStartMenuSC
Var AddQuickLaunchSC
Var AddDesktopSC
Var EnableAnonStats
Var SetDefaultBrowser
Var Admin
Var Metrics
Var StartTime
Var Status
Var CurrentPage 
Var MetricsCustom
Var MetricsEula
Var MetricsButton
Var P1Time
Var P2Time
Var P3Time
Var P4Time
Var P5Time
; Other included files may depend upon these includes!
; The following includes are provided by NSIS.
!include FileFunc.nsh
!include LogicLib.nsh
!include WinMessages.nsh
!include WordFunc.nsh
!include MUI.nsh

; WinVer.nsh was added in the same release that RequestExecutionLevel so check
; if ___WINVER__NSH___ is defined to determine if RequestExecutionLevel is
; available.
!include /NONFATAL WinVer.nsh
!ifdef ___WINVER__NSH___
  RequestExecutionLevel user
!else
  !warning "Installer will be created without Vista compatibility.$\n            \
            Upgrade your NSIS installation to at least version 2.22 to resolve."
!endif

!insertmacro GetOptions
!insertmacro GetParameters
!insertmacro GetSize
!insertmacro StrFilter
!insertmacro WordReplace
!insertmacro VersionCompare
!insertmacro GetFileVersion

; The following includes are custom.
!include branding.nsi
!include defines.nsi
!include common.nsh
!include locales.nsi
!include version.nsh

VIAddVersionKey "FileDescription" "${BrandShortName} Installer"

; Must be inserted before other macros that use logging
!insertmacro _LoggingCommon

!insertmacro AddDDEHandlerValues
!insertmacro ChangeMUIHeaderImage
!insertmacro CheckForFilesInUse
!insertmacro CleanUpdatesDir
!insertmacro CopyFilesFromDir
!insertmacro CreateRegKey
!insertmacro GetPathFromString
!insertmacro GetParent
!insertmacro IsHandlerForInstallDir
!insertmacro ManualCloseAppPrompt
!insertmacro RegCleanAppHandler
!insertmacro RegCleanMain
!insertmacro RegCleanUninstall
!insertmacro SetBrandNameVars
!insertmacro UnloadUAC
!insertmacro WriteRegStr2
!insertmacro WriteRegDWORD2

!include shared.nsh

; Helper macros for ui callbacks. Insert these after shared.nsh
!insertmacro CheckCustomCommon
!insertmacro InstallEndCleanupCommon
!insertmacro InstallOnInitCommon
!insertmacro InstallStartCleanupCommon
!insertmacro LeaveDirectoryCommon
!insertmacro OnEndCommon
!insertmacro PreDirectoryCommon

Name "${BrandFullName}"
OutFile "setup.exe"
InstallDirRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${BrandFullNameInternal} (${AppVersion})" "InstallLocation"
InstallDir "$PROGRAMFILES\${BrandFullName}\"
ShowInstDetails nevershow

ReserveFile welcome.ini
ReserveFile shortcuts.ini
ReserveFile finish.ini
ReserveFile horline.bmp
ReserveFile wizWaterMark.bmp 
ReserveFile License.exe

!define BST_UNCHECKED 0x0000
!define BST_CHECKED 0x0001
!define BST_INDETERMINATE 0x0002

################################################################################
# Modern User Interface - MUI

!define MUI_ABORTWARNING
!define MUI_ICON setup.ico
!define MUI_UNICON setup.ico
!define MUI_WELCOMEPAGE_TITLE_3LINES
!define MUI_HEADERIMAGE
!define MUI_HEADERIMAGE_RIGHT
!define MUI_WELCOMEFINISHPAGE_BITMAP wizWatermark.bmp
!define MUI_CUSTOMFUNCTION_GUIINIT checkupgrade
!define MUI_CUSTOMFUNCTION_ABORT custom.onUserAbort
; Use a right to left header image when the language is right to left
!ifdef ${AB_CD}_rtl
!define MUI_HEADERIMAGE_BITMAP_RTL wizHeaderRTL.bmp
!else
!define MUI_HEADERIMAGE_BITMAP wizHeader.bmp
!endif

/**
 * Installation Pages
 */
; Welcome Page
Page custom WelcomeIn WelcomeOut ""

; Custom Shortcuts Page

Page custom preShortcuts leaveShortcuts

; Select Install Directory Page

!define MUI_PAGE_CUSTOMFUNCTION_PRE preDirectory
!define MUI_PAGE_CUSTOMFUNCTION_LEAVE leaveDirectory
!define MUI_DIRECTORYPAGE_VERIFYONLEAVE
!insertmacro MUI_PAGE_DIRECTORY

; Start Menu Folder Page Configuration
!define MUI_PAGE_CUSTOMFUNCTION_PRE preStartMenu
!define MUI_PAGE_CUSTOMFUNCTION_LEAVE leaveStartMenu
!define MUI_STARTMENUPAGE_NODISABLE
!define MUI_STARTMENUPAGE_REGISTRY_ROOT "HKLM"
!define MUI_STARTMENUPAGE_REGISTRY_KEY "Software\Flock\${BrandFullNameInternal}\${AppVersion} (${AB_CD})\Main"
!define MUI_STARTMENUPAGE_REGISTRY_VALUENAME "Start Menu Folder"
!insertmacro MUI_PAGE_STARTMENU Application $StartMenuDir

; Install Files Page
!insertmacro MUI_PAGE_INSTFILES

; Finish Page
Page custom preFinish leaveFinish

!define MUI_PAGE_CUSTOMFUNCTION_PRE Completed
!insertmacro MUI_PAGE_FINISH
; Use the default dialog for IDD_VERIFY for a simple Banner
ChangeUI IDD_VERIFY "${NSISDIR}\Contrib\UIs\default.exe"

################################################################################
# Install Sections

; Cleanup operations to perform at the start of the installation.
Section "-InstallStartCleanup"
  StrCpy $CurrentPage "4"
  SetDetailsPrint both
  DetailPrint $(STATUS_CLEANUP)
  SetDetailsPrint none

  SetOutPath "$INSTDIR"
  ${StartInstallLog} "${BrandFullName}" "${AB_CD}" "${AppVersion}" "${GREVersion}"

  ; Delete the app exe to prevent launching the app while we are installing.
  ClearErrors
  ${DeleteFile} "$INSTDIR\${FileMainEXE}"
  ${If} ${Errors}
    ; If the user closed the application it can take several seconds for it to
    ; shut down completely. If the application is being used by another user we
    ; can rename the file and then delete is when the system is restarted.
    Sleep 5000
    ${DeleteFile} "$INSTDIR\${FileMainEXE}"
    ClearErrors
  ${EndIf}

  ${If} $InstallType == ${INSTALLTYPE_CUSTOM}
    ; Custom installs.
    ; If DOMi is installed and this install includes DOMi remove it from
    ; the installation directory. This will remove it if the user deselected
    ; DOMi on the components page.
    ${If} ${FileExists} "$INSTDIR\extensions\inspector@mozilla.org"
    ${AndIf} ${FileExists} "$EXEDIR\optional\extensions\inspector@mozilla.org"
      RmDir /r "$INSTDIR\extensions\inspector@mozilla.org"
    ${EndIf}
  ${EndIf}

  ; Remove the updates directory for Vista and above
  ${CleanUpdatesDir} "Flock\Browser"

  ${InstallStartCleanupCommon}
SectionEnd

Section "-Application" APP_IDX
  ${StartUninstallLog}

  SetDetailsPrint both
  DetailPrint $(STATUS_INSTALL_APP)
  SetDetailsPrint none

  ${LogHeader} "Installing Main Files"
  ${CopyFilesFromDir} "$EXEDIR\nonlocalized" "$INSTDIR" \
                      "$(ERROR_CREATE_DIRECTORY_PREFIX)" \
                      "$(ERROR_CREATE_DIRECTORY_SUFFIX)"

  ; Register DLLs
  ; XXXrstrong - AccessibleMarshal.dll can be used by multiple applications but
  ; is only registered for the last application installed. When the last
  ; application installed is uninstalled AccessibleMarshal.dll will no longer be
  ; registered. bug 338878
  ${LogHeader} "DLL Registration"
  ClearErrors
  RegDLL "$INSTDIR\AccessibleMarshal.dll"
  ${If} ${Errors}
    ${LogMsg} "** ERROR Registering: $INSTDIR\AccessibleMarshal.dll **"
  ${Else}
    ${LogUninstall} "DLLReg: \AccessibleMarshal.dll"
    ${LogMsg} "Registered: $INSTDIR\AccessibleMarshal.dll"
  ${EndIf}

  ; Write extra files created by the application to the uninstall.log so they
  ; will be removed when the application is uninstalled. To remove an empty
  ; directory write a bogus filename to the deepest directory and all empty
  ; parent directories will be removed.
  ${LogUninstall} "File: \components\compreg.dat"
  ${LogUninstall} "File: \components\xpti.dat"
  ${LogUninstall} "File: \.autoreg"
  ${LogUninstall} "File: \active-update.xml"
  ${LogUninstall} "File: \install.log"
  ${LogUninstall} "File: \install_status.log"
  ${LogUninstall} "File: \install_wizard.log"
  ${LogUninstall} "File: \updates.xml"

  SetDetailsPrint both
  DetailPrint $(STATUS_INSTALL_LANG)
  SetDetailsPrint none

  ${LogHeader} "Installing Localized Files"
  ${CopyFilesFromDir} "$EXEDIR\localized" "$INSTDIR" \
                      "$(ERROR_CREATE_DIRECTORY_PREFIX)" \
                      "$(ERROR_CREATE_DIRECTORY_SUFFIX)"

  ${LogHeader} "Adding Additional Files"
  ; Check if QuickTime is installed and copy the nsIQTScriptablePlugin.xpt from
  ; its plugins directory into the app's components directory.
  ClearErrors
  ReadRegStr $R0 HKLM "Software\Apple Computer, Inc.\QuickTime" "InstallDir"
  ${Unless} ${Errors}
    ${GetLongPath} $R0 "$R0"
    ${Unless} $R0 == ""
      ClearErrors
      GetFullPathName $R0 "$R0\Plugins\nsIQTScriptablePlugin.xpt"
      ${Unless} ${Errors}
        ${LogHeader} "Copying QuickTime Scriptable Component"
        CopyFiles /SILENT "$R0" "$INSTDIR\components"
        ${If} ${Errors}
          ${LogMsg} "** ERROR Installing File: $INSTDIR\components\nsIQTScriptablePlugin.xpt **"
        ${Else}
          ${LogMsg} "Installed File: $INSTDIR\components\nsIQTScriptablePlugin.xpt"
          ${LogUninstall} "File: \components\nsIQTScriptablePlugin.xpt"
        ${EndIf}
      ${EndUnless}
    ${EndUnless}
  ${EndUnless}
  ClearErrors

  ; Default for creating Start Menu folder and shortcuts
  ; (1 = create, 0 = don't create)
  ${If} $AddStartMenuSC == ""
    StrCpy $AddStartMenuSC "1"
  ${EndIf}

  ; Default for creating Quick Launch shortcut (1 = create, 0 = don't create)
  ${If} $AddQuickLaunchSC == ""
    StrCpy $AddQuickLaunchSC "1"
  ${EndIf}

  ; Default for creating Desktop shortcut (1 = create, 0 = don't create)
  ${If} $AddDesktopSC == ""
    StrCpy $AddDesktopSC "1"
  ${EndIf}

  ${LogHeader} "Adding Registry Entries"
  SetShellVarContext current  ; Set SHCTX to HKCU
  ${RegCleanMain} "Software\Flock"
  ${RegCleanUninstall}

  ClearErrors
  WriteRegStr HKLM "Software\Flock\InstallerTest" "InstallerTest" "Test"
  ${If} ${Errors}
    StrCpy $TmpVal "HKCU" ; used primarily for logging
  ${Else}
    SetShellVarContext all  ; Set SHCTX to HKLM
    DeleteRegKey HKLM "Software\Flock\InstallerTest"
    StrCpy $TmpVal "HKLM" ; used primarily for logging
    ${RegCleanMain} "Software\Flock"
    ${RegCleanUninstall}

    ReadRegStr $0 HKLM "Software\Flock" "CurrentVersion"
    ${If} "$0" != "${GREVersion}"
      WriteRegStr HKLM "Software\Flock" "CurrentVersion" "${GREVersion}"
    ${EndIf}
  ${EndIf}

  ${RemoveDeprecatedKeys}

  ; The previous installer adds several registry values to both HKLM and HKCU.
  ; We now try to add to HKLM and if that fails to HKCU

  ; The order that reg keys and values are added is important if you use the
  ; uninstall log to remove them on uninstall. When using the uninstall log you
  ; MUST add children first so they will be removed first on uninstall so they
  ; will be empty when the key is deleted. This allows the uninstaller to
  ; specify that only empty keys will be deleted.
  ${SetAppKeys}

  ; XXXrstrong - this should be set in shared.nsh along with "Create Quick
  ; Launch Shortcut" and Create Desktop Shortcut.
  StrCpy $0 "Software\Flock\${BrandFullNameInternal}\${AppVersion} (${AB_CD})\Uninstall"
  ${WriteRegDWORD2} $TmpVal "$0" "Create Start Menu Shortcut" $AddStartMenuSC 0

  ${FixClassKeys}
  ${UpdateProtocolHandlers}

  ; On install always add the FlockHTML and FlockURL keys.
  ; An empty string is used for the 5th param because FlockHTML and FlockURL
  ; are not protocol handlers.
  ${GetLongPath} "$INSTDIR\${FileMainEXE}" $8
  StrCpy $2 "$\"$8$\" -requestPending -osint -url $\"%1$\""
  StrCpy $3 "$\"%1$\",,0,0,,,,"

  ${AddDDEHandlerValues} "FlockHTML" "$2" "$8,1" "${AppRegName} Document" "" \
                         "${DDEApplication}" "$3" "WWW_OpenURL"

  ${AddDDEHandlerValues} "FlockURL" "$2" "$8,1" "${AppRegName} URL" "true" \
                         "${DDEApplication}" "$3" "WWW_OpenURL"

  ${FixShellIconHandler}

  ; The following keys should only be set if we can write to HKLM
  ${If} $TmpVal == "HKLM"
    ; Uninstall keys can only exist under HKLM on some versions of windows.
    ${SetUninstallKeys}

    ; Set the Start Menu Internet and Vista Registered App HKLM registry keys.
    ${SetStartMenuInternet}

    ; If we are writing to HKLM and create the quick launch and the desktop
    ; shortcuts set IconsVisible to 1 otherwise to 0.
    ${StrFilter} "${FileMainEXE}" "+" "" "" $R9
    StrCpy $0 "Software\Clients\StartMenuInternet\$R9\InstallInfo"
    ${If} $AddQuickLaunchSC == 1
    ${OrIf} $AddDesktopSC == 1
      WriteRegDWORD HKLM "$0" "IconsVisible" 1
    ${Else}
      WriteRegDWORD HKLM "$0" "IconsVisible" 0
    ${EndIf}
  ${EndIf}

  ; These need special handling on uninstall since they may be overwritten by
  ; an install into a different location.
  StrCpy $0 "Software\Microsoft\Windows\CurrentVersion\App Paths\${FileMainEXE}"
  ${WriteRegStr2} $TmpVal "$0" "" "$INSTDIR\${FileMainEXE}" 0
  ${WriteRegStr2} $TmpVal "$0" "Path" "$INSTDIR" 0

  StrCpy $0 "Software\Microsoft\MediaPlayer\ShimInclusionList\$R9"
  ${CreateRegKey} "$TmpVal" "$0" 0

  ; Check for the SetDefaultBrowser flag being set, and write the keys if needed.
  ${If} $SetDefaultBrowser == "1"
   ExecWait '"$INSTDIR\UNINSTALL\HELPER.EXE" /SetAsDefaultAppUser' 
   System::Call "shell32::SHChangeNotify(i, i, i, i) v (0x08000000, 0, 0, 0)"
  ${EndIf}

  !insertmacro MUI_STARTMENU_WRITE_BEGIN Application

  ; Create Start Menu shortcuts
  ${LogHeader} "Adding Shortcuts"
  ${If} $AddStartMenuSC == 1
    ${Unless} ${FileExists} "$SMPROGRAMS\$StartMenuDir"
      CreateDirectory "$SMPROGRAMS\$StartMenuDir"
      ${LogMsg} "Added Start Menu Directory: $SMPROGRAMS\$StartMenuDir"
    ${EndUnless}
    CreateShortCut "$SMPROGRAMS\$StartMenuDir\${BrandFullNameInternal}.lnk" "$INSTDIR\${FileMainEXE}" "" "$INSTDIR\${FileMainEXE}" 0
    ${LogUninstall} "File: $SMPROGRAMS\$StartMenuDir\${BrandFullNameInternal}.lnk"
    ${LogMsg} "Added Shortcut: $SMPROGRAMS\$StartMenuDir\${BrandFullNameInternal}.lnk"
    CreateShortCut "$SMPROGRAMS\$StartMenuDir\${BrandFullNameInternal} ($(SAFE_MODE)).lnk" "$INSTDIR\${FileMainEXE}" "-safe-mode" "$INSTDIR\${FileMainEXE}" 0
    ${LogUninstall} "File: $SMPROGRAMS\$StartMenuDir\${BrandFullNameInternal} ($(SAFE_MODE)).lnk"
    ${LogMsg} "Added Shortcut: $SMPROGRAMS\$StartMenuDir\${BrandFullNameInternal} ($(SAFE_MODE)).lnk"
    CreateShortCut "$SMPROGRAMS\$StartMenuDir\Uninstall ${BrandFullNameInternal}.lnk" "$INSTDIR\uninstall\helper.exe" "" "$INSTDIR\uninstall\helper.exe" 0
    ${LogUninstall} "File: $SMPROGRAMS\$StartMenuDir\Uninstall ${BrandFullNameInternal}.lnk"
    ${LogMsg} "Added Shortcut: $SMPROGRAMS\$StartMenuDir\Uninstall ${BrandFullNameInternal}.lnk"
  ${EndIf}

  ; perhaps use the uninstall keys
  ${If} $AddQuickLaunchSC == 1
    CreateShortCut "$QUICKLAUNCH\${BrandFullName}.lnk" "$INSTDIR\${FileMainEXE}" "" "$INSTDIR\${FileMainEXE}" 0
    ${LogUninstall} "File: $QUICKLAUNCH\${BrandFullName}.lnk"
    ${LogMsg} "Added Shortcut: $QUICKLAUNCH\${BrandFullName}.lnk"
  ${EndIf}

  ${If} $AddDesktopSC == 1
    CreateShortCut "$DESKTOP\${BrandFullName}.lnk" "$INSTDIR\${FileMainEXE}" "" "$INSTDIR\${FileMainEXE}" 0
    ${LogUninstall} "File: $DESKTOP\${BrandFullName}.lnk"
    ${LogMsg} "Added Shortcut: $DESKTOP\${BrandFullName}.lnk"
  ${EndIf}

  ${If} $EnableAnonStats == 0 ; Disable anonymous stats collecting.
    SetOutPath $INSTDIR\defaults\pref
    File /a "disablestats.js"
    SetOutPath $INSTDIR
    ${LogUninstall} "File: $INSTDIR\defaults\pref\disablestats.js"
    ${LogMsg} "Added File: $INSTDIR\defaults\pref\disablestats.js"
  ${EndIf}

  IfFileExists "$SYSDIR\Macromed\Flash\uninstall_plugin.exe" +1 flashinst
  ReadRegStr $R0 HKCU "Software\Macromedia\FlashPlayer" "FlashPlayerVersion"
  ${GetFileVersion} "$INSTDIR\install_flash_player.exe" $R1
  ${VersionCompare} "$R0" "$R1" "$R2"
  ${If} $R2 != 2
    ; the Flash that Flock installs is not newer
    goto done
  ${EndIf}
  ExecWait "$SYSDIR\Macromed\Flash\uninstall_plugin.exe"
  flashinst:
  ExecWait '"$INSTDIR\install_flash_player.exe" /S'
  ${LogUninstall} "File: $INSTDIR\install_flash_player.exe"
  ${LogMsg} "Added File: $INSTDIR\install_flash_player.exe"
  
  done:
  !insertmacro MUI_STARTMENU_WRITE_END
SectionEnd


; Not used
Section /o "Developer Tools" DOMI_IDX
  ${If} ${FileExists} "$EXEDIR\optional\extensions\inspector@mozilla.org"
    SetDetailsPrint both
    DetailPrint $(STATUS_INSTALL_OPTIONAL)
    SetDetailsPrint none

    ${RemoveDir} "$INSTDIR\extensions\inspector@mozilla.org"
    ClearErrors
    ${LogHeader} "Installing Developer Tools"
    ${CopyFilesFromDir} "$EXEDIR\optional\extensions\inspector@mozilla.org" \
                        "$INSTDIR\extensions\inspector@mozilla.org" \
                        "$(ERROR_CREATE_DIRECTORY_PREFIX)" \
                        "$(ERROR_CREATE_DIRECTORY_SUFFIX)"
  ${EndIf}
SectionEnd

; Cleanup operations to perform at the end of the installation.
Section "-InstallEndCleanup"
  Call CreateGUID
  Pop $0 ;contains GUID
  StrCpy $0 $0 -1 1
  IfFileExists $INSTDIR\defaults\pref\uuid.js skipuuid
  FileOpen $9 $INSTDIR\defaults\pref\uuid.js w
  FileWrite $9 'pref("flock.browser.uuid", "$0");'
  FileClose $9 
  skipuuid:
  SetDetailsPrint both
  DetailPrint "$(STATUS_CLEANUP)"
  SetDetailsPrint none
  ${LogHeader} "Updating Uninstall Log With Previous Uninstall Log"

  ; Refresh desktop icons
  System::Call "shell32::SHChangeNotify(i, i, i, i) v (0x08000000, 0, 0, 0)"

  ${InstallEndCleanupCommon}

  ; If we have to reboot give SHChangeNotify time to finish the refreshing
  ; the icons so the OS doesn't display the icons from helper.exe
  SetRebootFlag False

SectionEnd

################################################################################
# Helper Functions

Function CheckExistingInstall
  ; If there is a pending file copy from a previous uninstall don't allow
  ; installing until after the system has rebooted.
  IfFileExists "$INSTDIR\${FileMainEXE}.moz-upgrade" +1 +4
  MessageBox MB_YESNO "$(WARN_RESTART_REQUIRED_UPGRADE)" IDNO +2
  Reboot
  Quit

  ; If there is a pending file deletion from a previous uninstall don't allow
  ; installing until after the system has rebooted.
  IfFileExists "$INSTDIR\${FileMainEXE}.moz-delete" +1 +4
  MessageBox MB_YESNO "$(WARN_RESTART_REQUIRED_UNINSTALL)" IDNO +2
  Reboot
  Quit

  ${If} ${FileExists} "$INSTDIR\${FileMainEXE}"
    Banner::show /NOUNLOAD "$(BANNER_CHECK_EXISTING)"

    ${If} "$TmpVal" == "FoundMessageWindow"
      Sleep 5000
    ${EndIf}

    ${PushFilesToCheck}

    ; Store the return value in $TmpVal so it is less likely to be accidentally
    ; overwritten elsewhere.
    ${CheckForFilesInUse} $TmpVal

    Banner::destroy

    ${If} "$TmpVal" == "true"
      StrCpy $TmpVal "FoundMessageWindow"
      ${ManualCloseAppPrompt} "${WindowClass}" "$(WARN_MANUALLY_CLOSE_APP_INSTALL)"
      StrCpy $TmpVal "true"
    ${EndIf}
  ${EndIf}
FunctionEnd

Function LaunchApp
  ClearErrors
  ${GetParameters} $0
  ${GetOptions} "$0" "/UAC:" $1
  ${If} ${Errors}
    ${ManualCloseAppPrompt} "${WindowClass}" "$(WARN_MANUALLY_CLOSE_APP_LAUNCH)"
    Exec "$INSTDIR\${FileMainEXE}"
  ${Else}
    GetFunctionAddress $0 LaunchAppFromElevatedProcess
    UAC::ExecCodeSegment $0
    Exec "$INSTDIR\${FileMainEXE}"
  ${EndIf}
  Call HowLong
  Pop $P5Time
FunctionEnd

Function LaunchAppFromElevatedProcess
  ${ManualCloseAppPrompt} "${WindowClass}" "$(WARN_MANUALLY_CLOSE_APP_LAUNCH)"

  ; Find the installation directory when launching using GetFunctionAddress
  ; from an elevated installer since $INSTDIR will not be set in this installer
  ${StrFilter} "${FileMainEXE}" "+" "" "" $R9
  ReadRegStr $0 HKLM "Software\Clients\StartMenuInternet\$R9\DefaultIcon" ""
  ${GetPathFromString} "$0" $0
  ${GetParent} "$0" $1
  ; Set our current working directory to the application's install directory
  ; otherwise the 7-Zip temp directory will be in use and won't be deleted.
  SetOutPath "$1"
  Exec "$0"
FunctionEnd

################################################################################
# Language

!insertmacro MOZ_MUI_LANGUAGE 'baseLocale'
!verbose push
!verbose 3
!include "overrideLocale.nsh"
!include "customLocale.nsh"
!verbose pop

; Set this after the locale files to override it if it is in the locale
; using " " for BrandingText will hide the "Nullsoft Install System..." branding
BrandingText " "

################################################################################
# Page pre, show, and leave functions

Function preDirectory   
  StrCpy $CurrentPage "3"
  Call HowLong
  Pop $P3Time  
  ${If} $Admin == "false"
    StrCpy $INSTDIR "$APPDATA\Flock"
  ${EndIf}
  
  ${PreDirectoryCommon}
FunctionEnd

Function leaveDirectory
  ${LeaveDirectoryCommon} "$(WARN_DISK_SPACE)" "$(WARN_WRITE_ACCESS)"
  Call HowLong
  Pop $P3Time
FunctionEnd

Function preShortcuts
  StrCpy $CurrentPage "2"
  Call HowLong
  Pop $P2Time
  ${MUI_INSTALLOPTIONS_READ} $AddDesktopSC "shortcuts.ini" "Field 5" "State"
  ${MUI_INSTALLOPTIONS_READ} $AddStartMenuSC "shortcuts.ini" "Field 6" "State"
  ${MUI_INSTALLOPTIONS_READ} $AddQuickLaunchSC "shortcuts.ini" "Field 7" "State"
  ${MUI_INSTALLOPTIONS_READ} $EnableAnonStats "shortcuts.ini" "Field 10" "State"
  Call expresscheck
  ; ${CheckCustomCommon} ; We always want this to display.
  !insertmacro MUI_HEADER_TEXT "$(SHORTCUTS_PAGE_TITLE)" "$(SHORTCUTS_PAGE_SUBTITLE)"
  !insertmacro MUI_INSTALLOPTIONS_DISPLAY "shortcuts.ini"
FunctionEnd

Function leaveShortcuts
  ${MUI_INSTALLOPTIONS_READ} $0 "shortcuts.ini" "Settings" "State"
  ${If} $0 != 0
    Abort
  ${EndIf}
  ${MUI_INSTALLOPTIONS_READ} $SetDefaultBrowser "shortcuts.ini" "Field 3" "State"
  ${MUI_INSTALLOPTIONS_READ} $AddDesktopSC "shortcuts.ini" "Field 5" "State"
  ${MUI_INSTALLOPTIONS_READ} $AddStartMenuSC "shortcuts.ini" "Field 6" "State"
  ${MUI_INSTALLOPTIONS_READ} $AddQuickLaunchSC "shortcuts.ini" "Field 7" "State"
  ${MUI_INSTALLOPTIONS_READ} $EnableAnonStats "shortcuts.ini" "Field 10" "State"
  Call HowLong
  Pop $P2Time
FunctionEnd

Function preStartMenu
  Call HowLong
  Pop $P4Time
  ${If} $AddStartMenuSC != 1
  ${OrIf} $EXPRESSMODE != 0
    Abort
  ${EndIf}
FunctionEnd

Function leaveStartMenu
  ${If} $EXPRESSMODE != 1
    Call CheckExistingInstall
  ${EndIf}
  Call HowLong
  Pop $P4Time
FunctionEnd

; When we add an optional action to the finish page the cancel button is
; enabled. This disables it and leaves the finish button as the only choice.
Function preFinish
  StrCpy $CurrentPage "5"
  Call HowLong
  Pop $P5Time
  ${EndInstallLog} "${BrandFullName}"

  GetDlgItem $R2 $HWNDPARENT 2 ; disable CANCEL button
  EnableWindow $R2 0

  !insertmacro MUI_INSTALLOPTIONS_INITDIALOG "finish.ini"
  pop $HWND

  SetCtlColors $HWND "0x000000" "0xFFFFFF"
  ${MUI_INSTALLOPTIONS_READ} $2 "finish.ini" "Field 2" "HWND"
  SetCtlColors $2 '0x000000' '0xFFFFFF'
  ${MUI_INSTALLOPTIONS_READ} $2 "finish.ini" "Field 3" "HWND"
  SetCtlColors $2 '0x000000' '0xFFFFFF'
  ${MUI_INSTALLOPTIONS_READ} $2 "finish.ini" "Field 4" "HWND"
  SetCtlColors $2 '0x000000' '0xFFFFFF'
  ${MUI_INSTALLOPTIONS_READ} $2 "finish.ini" "Field 5" "HWND"
  SetCtlColors $2 '0x000000' '0xFFFFFF'
  ${MUI_INSTALLOPTIONS_READ} $2 "finish.ini" "Field 6" "HWND"
  SetCtlColors $2 '0x000000' '0xFFFFFF'
  ${MUI_INSTALLOPTIONS_READ} $2 "finish.ini" "Field 7" "HWND"
  SetCtlColors $2 '0x000000' '0xFFFFFF'
  ${MUI_INSTALLOPTIONS_READ} $2 "finish.ini" "Field 8" "HWND"
  SetCtlColors $2 '0x000000' '0xFFFFFF'
  ${MUI_INSTALLOPTIONS_READ} $2 "finish.ini" "Field 9" "HWND"
  SetCtlColors $2 '0x000000' '0xFFFFFF'
  ${MUI_INSTALLOPTIONS_READ} $2 "finish.ini" "Field 10" "HWND"
  SetCtlColors $2 '0x000000' '0xFFFFFF'

  ${MUI_INSTALLOPTIONS_READ} $2 "finish.ini" "Field 2" "HWND"

  CreateFont $1 "Arial" "14" "700"
  SendMessage $2 ${WM_SETFONT} $1 0

  ${MUI_INSTALLOPTIONS_READ} $2 "finish.ini" "Field 3" "HWND"  
  CreateFont $1 "Arial" "10" "700"
  SendMessage $2 ${WM_SETFONT} $1 0
  
  ${MUI_INSTALLOPTIONS_READ} $2 "finish.ini" "Field 8" "HWND"
  CreateFont $1 "Arial" "10" "" /ITALIC
  SendMessage $2 ${WM_SETFONT} $1 0

  !insertmacro MUI_INSTALLOPTIONS_SHOW
FunctionEnd

Function LeaveFinish

FunctionEnd

Function Completed
  Abort
FunctionEnd
; Just used for migrating Pre-FF installs.
Function CheckForExistingInstall
  ClearErrors
  ReadRegstr $0 HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${BrandFullNameInternal}" "UninstallString"
  IfErrors FinishGuiInit PossibleUpgrade

  PossibleUpgrade:
  StrLen $R2 "\uninst.exe"
  StrCpy $R1 $0 -$R2

  IfFileExists "$R1\uninst.exe" QueryUserToUpgrade FinishGuiInit

  QueryUserToUpgrade:
  MessageBox MB_OKCANCEL|MB_DEFBUTTON1|MB_ICONEXCLAMATION "$(UpgradeFlockNotice)" IDCANCEL NoClobberOld

  Exec '"$R1/uninst.exe" /S /DELETEPROFILE=NO /LAUNCHUNINSTALL=NO'
  Banner::Show /NOUNLOAD "$(RemovingPrevious)"
  Banner::getWindow /NOUNLOAD
  proc_check:
  Processes::FindProcess "Au_.exe"
  ${If} $R0 != 0
    Sleep 250
    Goto proc_check
  ${EndIf}
  Banner::destroy
  StrCpy $INSTDIR $R1
  Goto FinishGuiInit

  NoClobberOld:
  Abort

  FinishGuiInit:
FunctionEnd

################################################################################
# Initialization Functions

Function .onInit
  BringtoFront
  StrCpy $StartTime "0"
  StrCpy $MetricsButton "0"
  StrCpy $MetricsCustom "0"
  StrCpy $MetricsEula "0"
  time::_GetLocalTime /NOUNLOAD 
  Pop $StartTime 
  StrCpy $LANGUAGE 0
  StrCpy $Metrics ":"
  ${SetBrandNameVars} "$EXEDIR\localized\distribution\setup.ini"
  proc_check:
  sleep 500
  Processes::FindProcess "flock.exe"
  ${If} $R0 != 0
    MessageBox MB_OKCANCEL "$(CurrentlyRunninginstall)" IDOK proc_check IDCANCEL 0
    Abort "$(installCancel)"
  ${EndIf}
  ${InstallOnInitCommon} "$(WARN_UNSUPPORTED_MSG)"
  !insertmacro MUI_INSTALLOPTIONS_EXTRACT "welcome.ini"
  !insertmacro MUI_INSTALLOPTIONS_EXTRACT "shortcuts.ini"
  !insertmacro MUI_INSTALLOPTIONS_EXTRACT "finish.ini"
  File "/oname=$PLUGINSDIR\modern-wizard.bmp" "wizWaterMark.bmp"
  File "/oname=$PLUGINSDIR\horline.bmp" "horline.bmp"            
  File "/oname=$PLUGINSDIR\license.exe" "license.exe"
  !insertmacro createWelcomeINI
  !insertmacro createShortcutsINI
  !insertmacro createFinishINI

  ; There must always be nonlocalized and localized directories.
  ${GetSize} "$EXEDIR\nonlocalized\" "/S=0K" $R5 $R7 $R8
  ${GetSize} "$EXEDIR\localized\" "/S=0K" $R6 $R7 $R8
  IntOp $R8 $R5 + $R6
  SectionSetSize ${APP_IDX} $R8

  ${If} ${FileExists} "$EXEDIR\optional\extensions\inspector@mozilla.org"
    ; Set the section size for DOMi.
    ${GetSize} "$EXEDIR\optional\extensions\inspector@mozilla.org" "/S=0K" $0 $8 $9
    SectionSetSize ${DOMI_IDX} $0
  ${Else}
    ; Hide DOMi in the components page if it isn't available.
    SectionSetText ${DOMI_IDX} ""
  ${EndIf}

  ; Initialize $hHeaderBitmap to prevent redundant changing of the bitmap if
  ; the user clicks the back button
  StrCpy $hHeaderBitmap ""
FunctionEnd

Function Checkupgrade
  Call IsUserAdmin
  Pop $Admin  
  ${If} ${AtLeastWinVista}
    UAC::IsAdmin
    ; If the user is not an admin already
    ${If} "$0" != "1"
      ;MessageBox MB_OK "Not Admin : $0"
      ;UAC::SupportsUAC
      ; If the system supports UAC ( in $0, should be 1)
      ;UAC::GetElevationType
      UAC::RunElevated
      UAC::Unload
      Quit
    ${Else}
      ${GetParameters} $R9
      ${If} $R9 != ""
        ClearErrors
        ${GetOptions} "$R9" "/UAC:" $0
        ; If the command line contains /UAC then we need to initialize
        ; the UAC plugin to use UAC::ExecCodeSegment to execute code in
        ; the non-elevated context.
        ${Unless} ${Errors}
        UAC::RunElevated 
        ${EndUnless}
      ${EndIf}
    ${EndIf}
  ${EndIf}
  Call CheckForExistingInstall

FunctionEnd

Function .onGUIEnd
  ${OnEndCommon}
FunctionEnd

Function WelcomeIn ; EXPRESS/CUSTOM PAGE
  
  StrCpy $CurrentPage "1"
  !insertmacro MUI_HEADER_TEXT_PAGE "" "$(WelcomePageHeader)"
  !insertmacro MUI_INSTALLOPTIONS_INITDIALOG "welcome.ini"
  Pop $HWND

  FindWindow $1 "#32770" "" $HWNDPARENT
  SetCtlColors $1 "0x000000" "0xFFFFFF"
  GetDlgItem $2 $1 1201
  SetCtlColors $2 '0x000000' '0xFFFFFF'
  GetDlgItem $2 $1 1202
  SetCtlColors $2 '0x000000' '0xFFFFFF'
 
  GetDlgItem $2 $1 1203
  SetCtlColors $2 '0x000000' '0xFFFFFF'

  GetDlgItem $2 $1 1204
  SetCtlColors $2 '0x000000' '0xFFFFFF'
  GetDlgItem $2 $1 1205
  SetCtlColors $2 '0x000000' '0xFFFFFF'
  GetDlgItem $2 $1 1206
  SetCtlColors $2 '0x000000' '0xFFFFFF'
  GetDlgItem $2 $1 1207
  SetCtlColors $2 '0x000000' '0xFFFFFF'
  GetDlgItem $2 $1 1208
  SetCtlColors $2 '0x000000' '0xFFFFFF'
  GetDlgItem $2 $1 1209
  SetCtlColors $2 '0x000000' '0xFFFFFF'
  GetDlgItem $2 $1 1210
  SetCtlColors $2 '0x000000' '0xFFFFFF'
  GetDlgItem $2 $1 1211
  SetCtlColors $2 '0x000000' '0xFFFFFF'
  GetDlgItem $2 $1 1212
  SetCtlColors $2 '0x000000' '0xFFFFFF'  
  
  FindWindow $1 "#32770" "" $HWNDPARENT ; Express
  GetDlgItem $2 $1 1202
  CreateFont $1 "Arial" "9" "700"
  SendMessage $2 ${WM_SETFONT} $1 0  
  
  FindWindow $1 "#32770" "" $HWNDPARENT ; Custom
  GetDlgItem $2 $1 1203
  CreateFont $1 "Arial" "9" "700"
  SendMessage $2 ${WM_SETFONT} $1 0

  FindWindow $1 "#32770" "" $HWNDPARENT ; text for express
  GetDlgItem $2 $1 1210
  CreateFont $1 "Arial" "8" "0"
  SendMessage $2 ${WM_SETFONT} $1 0
    
  GetDlgItem $1 $HWNDPARENT 1038 ; Title
  CreateFont $2 "Arial" "12" "600"
  SendMessage $1 ${WM_SETFONT} $2 0
  
  FindWindow $1 "#32770" "" $HWNDPARENT
  SetCtlColors $1 '' '0xFFFFFF'
  
  ReadINIStr $R1 "$PLUGINSDIR\welcome.ini" "Field 7" "State"
  ${If} $R1 != 1
    GetDlgItem $R0 $HWNDPARENT 1 ; disable Next button
    EnableWindow $R0 0
  ${Else}
    GetDlgItem $R0 $HWNDPARENT 1 ; enable Next button
    EnableWindow $R0 1
  ${EndIf}

  !insertmacro MUI_INSTALLOPTIONS_SHOW
  pop $R0
FunctionEnd

Function WelcomeOut

  ; Metrics
  GetDlgItem $1 $HWNDPARENT 1038 ; Title
  CreateFont $2 "Arial" "8" "600"
  SendMessage $1 ${WM_SETFONT} $2 0 
  ReadINIStr $MetricsCustom "$PLUGINSDIR\welcome.ini" "Field 4" "State" ;  Express or Custom
  ReadINIStr $MetricsEula "$PLUGINSDIR\welcome.ini" "Field 7" "State" ;  Express or Custom
  ReadINIStr $MetricsButton "$PLUGINSDIR\welcome.ini" "Settings" "State"
  StrCpy $SetDefaultBrowser "1"
  ReadINIStr $0 "$PLUGINSDIR\welcome.ini" "Settings" "State"
  ${Select} $0
  ${Case} "0" ; next button
  goto validate
  ${Case} "3" ; Express mode
    GetDlgItem $1 $hwnd 1202 ; PathRequest control (1200 + field 7 - 1)
    SendMessage $1 ${BM_SETCHECK} ${BST_CHECKED} 0
    GetDlgItem $1 $hwnd 1203 ; PathRequest control (1200 + field 7 - 1)
    SendMessage $1 ${BM_SETCHECK} ${BST_UNCHECKED} 0
    WriteINIStr "$PLUGINSDIR\welcome.ini" "Field 3" "State" "1"
    WriteINIStr "$PLUGINSDIR\welcome.ini" "Field 4" "State" "0"
  ${Case} "4" ; Express mode
    GetDlgItem $1 $hwnd 1202 ; PathRequest control (1200 + field 7 - 1)
    SendMessage $1 ${BM_SETCHECK} ${BST_UNCHECKED} 0
    GetDlgItem $1 $hwnd 1203 ; PathRequest control (1200 + field 7 - 1)
    SendMessage $1 ${BM_SETCHECK} ${BST_CHECKED} 0
    WriteINIStr "$PLUGINSDIR\welcome.ini" "Field 3" "State" "0"
    WriteINIStr "$PLUGINSDIR\welcome.ini" "Field 3" "State" "1"
  ${Case} "7" ; Accept Checkbox
    ReadINIStr $R1 "$PLUGINSDIR\welcome.ini" "Field 7" "State"
    ${If} $R1 != 1
      GetDlgItem $R0 $HWNDPARENT 1 ; disable Next button
      EnableWindow $R0 0
    ${Else}
      GetDlgItem $R0 $HWNDPARENT 1 ; enable Next button
      EnableWindow $R0 1
    ${EndIf}
  ${Case} "8" ; Eula Button
    ExecWait "$PLUGINSDIR\license.exe"
  ${CaseElse}
  ${EndSelect}
  Abort ; Return to the page

validate:
  ReadINIStr $EXPRESSMODE "$PLUGINSDIR\welcome.ini" "Field 3" "State" ;  Express or Custom
  Call HowLong
  Pop $P1Time

FunctionEnd

Function expresscheck
  ${IF} $EXPRESSMODE == 1
    abort
  ${EndIf}
FunctionEnd

; Usage:
;   Call IsUserAdmin
;   Pop $R0   ; at this point $R0 is "true" or "false"
;
Function IsUserAdmin
  Push $R0
  Push $R1
  Push $R2

  ClearErrors
  UserInfo::GetName
  IfErrors Win9x
  Pop $R1
  UserInfo::GetAccountType
  Pop $R2
  
  ${If} $R2 == "Admin"
    StrCpy $R0 "true"
  ${ElseIf} $R2 != ""
    StrCpy $R0 "false"
  ${Else}
    Win9x:
    StrCpy $R0 "true"
  ${EndIf}

  Pop $R2
  Pop $R1
  Exch $R0
FunctionEnd

; How long since the installer started running?
; Pop $VAR to find out
Function HowLong
  time::_GetLocalTime 
  Pop $R0 
  time::_MathTime /NOUNLOAD "second($R0) - second($StartTime) = second"
FunctionEnd

Function CallmeBack
  ; Send the instrumentation data back to the server.
  Call HowLong
  Pop $R1
  inetc::get /CAPTION " " /USERAGENT "FlockINSTALL" /BANNER "Please Wait.." /NOCANCEL /END "http://metrics.flock.com/flock-installer?V${AppVersion}&$STATUS&$R1&$Metrics" "$PLUGINSDIR\temp"

  Pop $0
  time::_unload
  ;MessageBox MB_OK "EXIT CODE: [$STATUS] TIME IN INSTALLER: [$R1] METRICS DATA: [$METRICS]"
FunctionEnd


Function BuildString
  Call HowLong
  Pop $0
  StrCpy $Metrics "EXIT$CURRENTPAGE:&P1&C$MetricsCustom&E$MetricsEula&B$MetricsButton&T$P1Time"
  
  ${IF} $P2Time != ""
  ${MUI_INSTALLOPTIONS_READ} $AddDesktopSC "shortcuts.ini" "Field 5" "State"
  ${MUI_INSTALLOPTIONS_READ} $AddStartMenuSC "shortcuts.ini" "Field 6" "State"
  ${MUI_INSTALLOPTIONS_READ} $AddQuickLaunchSC "shortcuts.ini" "Field 7" "State"
  ${MUI_INSTALLOPTIONS_READ} $EnableAnonStats "shortcuts.ini" "Field 10" "State"
  ${MUI_INSTALLOPTIONS_READ} $SetDefaultBrowser "shortcuts.ini" "Field 3" "State"
    StrCpy $Metrics "$Metrics:&P2&D$SetDefaultBrowser&K$AddDesktopSC&S$AddStartMenuSC&U$AddQuickLaunchSC&R$EnableAnonStats&T$P2Time"
  ${EndIf}
  ${IF} $P3Time != ""
    StrCpy $Metrics "$Metrics:&P3&T$P3Time"
  ${EndIf}
    ${IF} $P4Time != ""
    StrCpy $Metrics "$Metrics:&P4&T$P4Time"
  ${EndIf}
    ${IF} $P5Time != ""
    StrCpy $Metrics "$Metrics:&P5&T$P5Time"
  ${EndIf}

FunctionEnd
Function .onInstFailed
  StrCpy $Status "FAILED"
  Call BuildString
  Call CallmeBack
  UAC::Unload ;Must call unload!
FunctionEnd

Function .onInstSuccess
  StrCpy $Status "SUCCESS"
  Call BuildString
  Call CallmeBack
  ${MUI_INSTALLOPTIONS_READ} $R0 "finish.ini" "Field 9" "State"
  ${If} $R0 != "0"
    Call LaunchApp
  ${EndIf}
  UAC::Unload ;Must call unload!
FunctionEnd

Function Custom.onUserAbort
  StrCpy $Status "ABORT"
  Call BuildString
  Call CallmeBack
  UAC::Unload ;Must call unload!
FunctionEnd

Function .onVerifyInstDir

FunctionEnd