# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is the Mozilla Installer code.
#
# The Initial Developer of the Original Code is Mozilla Foundation
# Portions created by the Initial Developer are Copyright (C) 2006
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#  Robert Strong <robert.bugzilla@gmail.com>
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

# Required Plugins:
# AppAssocReg http://nsis.sourceforge.net/Application_Association_Registration_plug-in
# ShellLink   http://nsis.sourceforge.net/ShellLink_plug-in
# UAC         http://nsis.sourceforge.net/UAC_plug-in

; Set verbosity to 3 (e.g. no script) to lessen the noise in the build logs
!verbose 3

; 7-Zip provides better compression than the lzma from NSIS so we add the files
; uncompressed and use 7-Zip to create a SFX archive of it
SetDatablockOptimize on
SetCompress off
CRCCheck on

RequestExecutionLevel user

!addplugindir ./

; USE_UAC_PLUGIN is temporary until all applications have been updated to use
; the UAC plugin
!define USE_UAC_PLUGIN

; prevents compiling of the reg write logging.
!define NO_LOG

Var TmpVal
Var IEVERSION
Var Starttime
Var Status
Var Metrics
Var GUID
; Other included files may depend upon these includes!
; The following includes are provided by NSIS.
!include FileFunc.nsh
!include LogicLib.nsh
!include MUI.nsh
!include TextFunc.nsh
!include WinMessages.nsh
!include WordFunc.nsh

!insertmacro StrFilter
!insertmacro WordReplace

!insertmacro un.GetParent
!insertmacro un.LineFind
!insertmacro un.TrimNewLines

; The following includes are custom.
!include branding.nsi
!include defines.nsi
!include common.nsh
!include locales.nsi
!include version.nsh
!include time.nsh
; This is named BrandShortName helper because we use this for software update
; post update cleanup.
VIAddVersionKey "FileDescription" "${BrandShortName} Helper"

!insertmacro AddDDEHandlerValues
!insertmacro CleanVirtualStore
!insertmacro GetLongPath
!insertmacro GetPathFromString
!insertmacro IsHandlerForInstallDir
!insertmacro RegCleanAppHandler
!insertmacro RegCleanMain
!insertmacro RegCleanUninstall
!insertmacro SetBrandNameVars
!insertmacro UnloadUAC
!insertmacro WriteRegDWORD2
!insertmacro WriteRegStr2

!insertmacro un.ChangeMUIHeaderImage
!insertmacro un.CheckForFilesInUse
!insertmacro un.CleanUpdatesDir
!insertmacro un.CleanVirtualStore
!insertmacro un.DeleteRelativeProfiles
!insertmacro un.GetLongPath
!insertmacro un.GetSecondInstallPath
!insertmacro un.ManualCloseAppPrompt
!insertmacro un.ParseUninstallLog
!insertmacro un.RegCleanAppHandler
!insertmacro un.RegCleanFileHandler
!insertmacro un.RegCleanMain
!insertmacro un.RegCleanUninstall
!insertmacro un.RegCleanProtocolHandler
!insertmacro un.RemoveQuotesFromPath
!insertmacro un.SetBrandNameVars

!include shared.nsh

; Helper macros for ui callbacks. Insert these after shared.nsh
!insertmacro OnEndCommon
!insertmacro UninstallOnInitCommon

!insertmacro un.OnEndCommon

Name "${BrandFullName}"
OutFile "helper.exe"
InstallDirRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${BrandFullNameInternal} (${AppVersion})" "InstallLocation"
InstallDir "$PROGRAMFILES\${BrandFullName}"
ShowUnInstDetails nevershow

################################################################################
# Modern User Interface - MUI

!define MUI_ABORTWARNING
!define MUI_ICON setup.ico
!define MUI_UNICON setup.ico
!define MUI_WELCOMEPAGE_TITLE_3LINES
!define MUI_HEADERIMAGE
!define MUI_HEADERIMAGE_RIGHT
!define MUI_UNWELCOMEFINISHPAGE_BITMAP wizWatermark.bmp
!define MUI_CUSTOMFUNCTION_ABORT custom.onUserAbort

; Use a right to left header image when the language is right to left
!ifdef ${AB_CD}_rtl
!define MUI_HEADERIMAGE_BITMAP_RTL wizHeaderRTL.bmp
!else
!define MUI_HEADERIMAGE_BITMAP wizHeader.bmp
!endif

/**
 * Uninstall Pages
 */
; Welcome Page
!define MUI_PAGE_CUSTOMFUNCTION_PRE un.preWelcome
!define MUI_PAGE_CUSTOMFUNCTION_LEAVE un.leaveWelcome
!insertmacro MUI_UNPAGE_WELCOME

; Uninstall Confirm Page
UninstPage custom un.preConfirm un.leaveConfirm

; Remove Files Page
!insertmacro MUI_UNPAGE_INSTFILES

; Finish Page

; Don't setup the survey controls, functions, etc. when the application has
; defined NO_UNINSTALL_SURVEY
!ifndef NO_UNINSTALL_SURVEY
!define MUI_PAGE_CUSTOMFUNCTION_PRE un.preFinish
;!define MUI_FINISHPAGE_SHOWREADME_NOTCHECKED
!define MUI_FINISHPAGE_SHOWREADME ""
!define MUI_FINISHPAGE_SHOWREADME_TEXT $(SURVEY_TEXT)
!define MUI_FINISHPAGE_SHOWREADME_FUNCTION un.Survey
!endif

!insertmacro MUI_UNPAGE_FINISH

; Use the default dialog for IDD_VERIFY for a simple Banner
ChangeUI IDD_VERIFY "${NSISDIR}\Contrib\UIs\default.exe"

################################################################################
# Install Sections
; Empty section required for the installer to compile as an uninstaller
Section ""
SectionEnd

################################################################################
# Uninstall Sections

Section "Uninstall"
  SetDetailsPrint textonly
  DetailPrint $(STATUS_UNINSTALL_MAIN)
  SetDetailsPrint none

  ; Delete the app exe to prevent launching the app while we are uninstalling.
  ClearErrors
  ${DeleteFile} "$INSTDIR\${FileMainEXE}"
  ${If} ${Errors}
    ; If the user closed the application it can take several seconds for it to
    ; shut down completely. If the application is being used by another user we
    ; can still delete the files when the system is restarted. 
    Sleep 5000
    ${DeleteFile} "$INSTDIR\${FileMainEXE}"
    ClearErrors
  ${EndIf}

  StrCpy $Metrics "PROFILE"
  ${MUI_INSTALLOPTIONS_READ} $0 "unconfirm.ini" "Field 3" "State"
  ${If} "$0" == "1"
    ${un.DeleteRelativeProfiles} "Flock\Browser"
    StrCpy $Metrics "NOPROFILE"  
  ${EndIf}

  SetShellVarContext current  ; Set SHCTX to HKCU
  ${un.RegCleanMain} "Software\Flock"
  ${un.RegCleanUninstall}

  ClearErrors
  WriteRegStr HKLM "Software\Flock\InstallerTest" "InstallerTest" "Test"
  ${If} ${Errors}
    StrCpy $TmpVal "HKCU" ; used primarily for logging
  ${Else}
    SetShellVarContext all  ; Set SHCTX to HKLM
    DeleteRegKey HKLM "Software\Flock\InstallerTest"
    StrCpy $TmpVal "HKLM" ; used primarily for logging
    ${un.RegCleanMain} "Software\Flock"
    ${un.RegCleanUninstall}
  ${EndIf}

  ${un.RegCleanAppHandler} "FlockURL"
  ${un.RegCleanAppHandler} "FlockHTML"
  ${un.RegCleanProtocolHandler} "ftp"
  ${un.RegCleanProtocolHandler} "http"
  ${un.RegCleanProtocolHandler} "https"

  ClearErrors
  ReadRegStr $R9 HKCR "FlockHTML" ""
  ; Don't clean up the file handlers if the FlockHTML key still exists since
  ; there should be a second installation that may be the default file handler
  ${If} ${Errors}
    ${un.RegCleanFileHandler}  ".htm"   "FlockHTML"
    ${un.RegCleanFileHandler}  ".html"  "FlockHTML"
    ${un.RegCleanFileHandler}  ".shtml" "FlockHTML"
    ${un.RegCleanFileHandler}  ".xht"   "FlockHTML"
    ${un.RegCleanFileHandler}  ".xhtml" "FlockHTML"
  ${EndIf}

  SetShellVarContext all  ; Set SHCTX to HKLM
  ${un.GetSecondInstallPath} "Software\Flock" $R9
  ${If} $R9 == "false"
    SetShellVarContext current  ; Set SHCTX to HKCU
    ${un.GetSecondInstallPath} "Software\Flock" $R9
  ${EndIf}

  StrCpy $0 "Software\Clients\StartMenuInternet\${FileMainEXE}\shell\open\command"
  ReadRegStr $R1 HKLM "$0" ""
  ${un.RemoveQuotesFromPath} "$R1" $R1
  ${un.GetParent} "$R1" $R1
  GetFullPathName /SHORT $R2 $INSTDIR
  GetFullPathName /SHORT $R1 $R1  
  ;MessageBox MB_OK "Startmenuinternet | $R1 Match $R2 "
  ; Only remove the StartMenuInternet key if it refers to this install location.
  ; The StartMenuInternet registry key is independent of the default browser
  ; settings. The XPInstall base un-installer always removes this key if it is
  ; uninstalling the default browser and it will always replace the keys when
  ; installing even if there is another install of flock that is set as the
  ; default browser. Now the key is always updated on install but it is only
  ; removed if it refers to this install location.
  ${If} "$R2" == "$R1"
    DeleteRegKey HKLM "Software\Clients\StartMenuInternet\${FileMainEXE}"
    DeleteRegValue HKLM "Software\RegisteredApplications" "${AppRegName}"

    ; If we own the install, and we're default startmenuInternet app, reset to IE.
    StrCpy $0 "Software\Clients\StartMenuInternet"
    ReadRegStr $R1 HKLM "$0" ""
    ${If} $R1 == "FLOCK.EXE"
    
    ; Set IE as default in "Set Program Access and Defaults"
    WriteRegStr HKLM "SOFTWARE\Clients\StartMenuInternet" "" "IEXPLORE.EXE"
    SendMessage ${HWND_BROADCAST} ${WM_SETTINGCHANGE} 0 "Software\Clients\StartMenuInternet" $R0
  ${If} $TmpVal == "HKLM"
    ; Restore file and protocol handlers.
    ; Read Current IE Version
    ReadRegStr $IEVERSION HKLM "Software\Microsoft\Internet Explorer" "Version"
    Strcpy $IEVERSION $IEVERSION 1
    ${Select} $IEVERSION 
    ${Case} "6"
    ; Verified keys for IE6, windows xp.
    WriteRegStr HKLM "SOFTWARE\Classes\.htm" "" "htmlfile"
    WriteRegStr HKLM "SOFTWARE\Classes\.html" "" "htmlfile"
    WriteRegStr HKLM "SOFTWARE\Classes\ftp\DefaultIcon" "" "%SystemRoot%\system32\url.dll,0"
    WriteRegStr HKLM "SOFTWARE\Classes\GOPHER\DefaultIcon" "" "%SystemRoot%\system32\url.dll,0"
    WriteRegStr HKLM "SOFTWARE\Classes\HTTP\DefaultIcon" "" "%SystemRoot%\system32\url.dll,0"
    WriteRegStr HKLM "SOFTWARE\Classes\https\DefaultIcon" "" "%SystemRoot%\system32\url.dll,0"  
    WriteRegStr HKLM "SOFTWARE\Classes\ftp\shell\open\command" "" '"C:\Program Files\Internet Explorer\IEXPLORE.EXE" %1'
    WriteRegStr HKLM "SOFTWARE\Classes\ftp\shell\open\ddeexec" "" '"%1",,-1,0,,,,'
    WriteRegStr HKLM "SOFTWARE\Classes\ftp\shell\open\ddeexec\Application" "" "IExplore"
    WriteRegStr HKLM "SOFTWARE\Classes\GOPHER\shell\open\command" "" '"C:\Program Files\Internet Explorer\IEXPLORE.EXE" -nohome'
    WriteRegStr HKLM "SOFTWARE\Classes\GOPHER\shell\open\ddeexec" "" '"%1",,-1,0,,,,'
    WriteRegStr HKLM "SOFTWARE\Classes\GOPHER\shell\open\ddeexec\Application" "" "IExplore"
    WriteRegStr HKLM "SOFTWARE\Classes\HTTP\shell\open\command" "" '"C:\Program Files\Internet Explorer\IEXPLORE.EXE" -nohome'
    WriteRegStr HKLM "SOFTWARE\Classes\HTTP\shell\open\ddeexec" "" '"%1",,-1,0,,,,'
    WriteRegStr HKLM "SOFTWARE\Classes\HTTP\shell\open\ddeexec\Application" "" "IExplore"
    WriteRegStr HKLM "SOFTWARE\Classes\https\shell\open\command" "" '"C:\Program Files\Internet Explorer\IEXPLORE.EXE" -nohome'
    WriteRegStr HKLM "SOFTWARE\Classes\https\shell\open\ddeexec" "" '"%1",,-1,0,,,,'
    WriteRegStr HKLM "SOFTWARE\Classes\https\shell\open\ddeexec\Application" "" "IExplore"    
  ${Case} "7"
    WriteRegStr HKLM "SOFTWARE\Classes\.htm" "" "htmlfile"
    WriteRegStr HKLM "SOFTWARE\Classes\.html" "" "htmlfile"
    WriteRegStr HKLM "SOFTWARE\Classes\ftp\DefaultIcon" "" "%SystemRoot%\system32\url.dll,0"
    WriteRegStr HKLM "SOFTWARE\Classes\GOPHER\DefaultIcon" "" "%SystemRoot%\system32\url.dll,0"
    WriteRegStr HKLM "SOFTWARE\Classes\HTTP\DefaultIcon" "" "%SystemRoot%\system32\url.dll,0"
    WriteRegStr HKLM "SOFTWARE\Classes\https\DefaultIcon" "" "%SystemRoot%\system32\url.dll,0"  
    WriteRegStr HKLM "SOFTWARE\Classes\ftp\shell\open\command" "" '"C:\Program Files\Internet Explorer\IEXPLORE.EXE" %1'
    WriteRegStr HKLM "SOFTWARE\Classes\ftp\shell\open\ddeexec" "" '"%1",,-1,0,,,,'
    WriteRegStr HKLM "SOFTWARE\Classes\ftp\shell\open\ddeexec\Application" "" "IExplore"
    WriteRegStr HKLM "SOFTWARE\Classes\GOPHER\shell\open\command" "" '"C:\Program Files\Internet Explorer\IEXPLORE.EXE" -nohome'
    WriteRegStr HKLM "SOFTWARE\Classes\GOPHER\shell\open\ddeexec" "" '"%1",,-1,0,,,,'
    WriteRegStr HKLM "SOFTWARE\Classes\GOPHER\shell\open\ddeexec\Application" "" "IExplore"
    WriteRegStr HKLM "SOFTWARE\Classes\HTTP\shell\open\command" "" '"C:\Program Files\Internet Explorer\IEXPLORE.EXE" -nohome'
    WriteRegStr HKLM "SOFTWARE\Classes\HTTP\shell\open\ddeexec" "" '"%1",,-1,0,,,,'
    WriteRegStr HKLM "SOFTWARE\Classes\HTTP\shell\open\ddeexec\Application" "" "IExplore"
    WriteRegStr HKLM "SOFTWARE\Classes\https\shell\open\command" "" '"C:\Program Files\Internet Explorer\IEXPLORE.EXE" -nohome'
    WriteRegStr HKLM "SOFTWARE\Classes\https\shell\open\ddeexec" "" '"%1",,-1,0,,,,'
    WriteRegStr HKLM "SOFTWARE\Classes\https\shell\open\ddeexec\Application" "" "IExplore"    

    ; Handle IE7 on XP keys.
    ${CaseElse}
    ${EndSelect} 
    ${EndIf}
  
    ${EndIf}   
  ${EndIf} 
  StrCpy $0 "Software\Microsoft\Windows\CurrentVersion\App Paths\${FileMainEXE}"
  ${If} $R9 == "false"
    DeleteRegKey HKLM "$0"
    DeleteRegKey HKCU "$0"
    StrCpy $0 "Software\Microsoft\MediaPlayer\ShimInclusionList\${FileMainEXE}"
    DeleteRegKey HKLM "$0"
    DeleteRegKey HKCU "$0"
    StrCpy $0 "Software\Classes\MIME\Database\Content Type\application/x-xpinstall;app=flock"
    DeleteRegKey HKLM "$0"
    DeleteRegKey HKCU "$0"
  ${Else}
    ReadRegStr $R1 HKLM "$0" ""
    ${un.RemoveQuotesFromPath} "$R1" $R1
    ${un.GetParent} "$R1" $R1
    GetFullPathName /SHORT $R2 $INSTDIR
    GetFullPathName /SHORT $R1 $R1  
    ${If} "$R2" == "$R1"
      WriteRegStr HKLM "$0" "" "$R9"
      ${un.GetParent} "$R9" $R1
      WriteRegStr HKLM "$0" "Path" "$R1"
    ${EndIf}
  ${EndIf}

  ; Remove directories and files we always control before parsing the uninstall
  ; log so empty directories can be removed.
  ${If} ${FileExists} "$INSTDIR\updates"
    RmDir /r /REBOOTOK "$INSTDIR\updates"
  ${EndIf}
  ${If} ${FileExists} "$INSTDIR\defaults\shortcuts"
    RmDir /r /REBOOTOK "$INSTDIR\defaults\shortcuts"
  ${EndIf}
  ${If} ${FileExists} "$INSTDIR\distribution"
    RmDir /r /REBOOTOK "$INSTDIR\distribution"
  ${EndIf}
  ${If} ${FileExists} "$INSTDIR\removed-files"
    Delete /REBOOTOK "$INSTDIR\removed-files"
  ${EndIf}

  ; Remove the updates directory for Vista and above
  ${un.CleanUpdatesDir} "Flock\Browser"

  ; Remove files that may be left behind by the application in the
  ; VirtualStore directory.
  ${un.CleanVirtualStore}

  ; Parse the uninstall log to unregister dll's and remove all installed
  ; files / directories this install is responsible for.
  ${un.ParseUninstallLog}

  ; Remove the uninstall directory that we control
  RmDir /r /REBOOTOK "$INSTDIR\uninstall"
  
  ; Remove the flock folder
  RmDir /r /REBOOTOK "$INSTDIR\Flock"

  ; Remove the installation directory if it is empty
  ${RemoveDir} "$INSTDIR"

  ; Remove files that may be left behind by the application in the
  ; VirtualStore directory.
  ${un.CleanVirtualStore}

  ; Handle Vista specific keys
  ; Use the UAC to run this command as the current user
  ${If} ${AtLeastWinVista}
  ${un.RegCleanFileHandler}  ".htm"   "FlockHTML"
  ${un.RegCleanFileHandler}  ".html"  "FlockHTML"
  ${un.RegCleanFileHandler}  ".shtml" "FlockHTML"
  ${un.RegCleanFileHandler}  ".xht"   "FlockHTML"
  ${un.RegCleanFileHandler}  ".xhtml" "FlockHTML"
  GetFunctionAddress $0 un.WriteVistaLimitedKeys
  UAC::ExecCodeSegment $0
  ${Endif}  
  
  ; If flock.exe was successfully deleted yet we still need to restart to
  ; remove other files create a dummy flock.exe.moz-delete to prevent the
  ; installer from allowing an install without restart when it is required
  ; to complete an uninstall.
  ${If} ${RebootFlag}
    ${Unless} ${FileExists} "$INSTDIR\${FileMainEXE}.moz-delete"
      FileOpen $0 "$INSTDIR\${FileMainEXE}.moz-delete" w
      FileWrite $0 "Will be deleted on restart"
      Delete /REBOOTOK "$INSTDIR\${FileMainEXE}.moz-delete"
      FileClose $0
    ${EndUnless}
  ${EndIf}

  ; Refresh desktop icons otherwise the start menu internet item won't be
  ; removed and other ugly things will happen like recreation of the app's
  ; clients registry key by the OS under some conditions.
  System::Call "shell32::SHChangeNotify(i, i, i, i) v (0x08000000, 0, 0, 0)"
SectionEnd

################################################################################
# Helper Functions

Function un.WriteVistaLimitedKeys
  ${un.RegCleanFileHandler}  ".htm"   "FlockHTML"
  ${un.RegCleanFileHandler}  ".html"  "FlockHTML"
  ${un.RegCleanFileHandler}  ".shtml" "FlockHTML"
  ${un.RegCleanFileHandler}  ".xht"   "FlockHTML"
  ${un.RegCleanFileHandler}  ".xhtml" "FlockHTML"
  ;MessageBox MB_OK "VISTA"
  WriteRegStr HKCU "Software\Microsoft\Windows\Shell\Associations\UrlAssociations\ftp\UserChoice" "Progid"  "IE.FTP"
  WriteRegStr HKCU "Software\Microsoft\Windows\Shell\Associations\UrlAssociations\http\UserChoice" "Progid"  "IE.HTTP"
  WriteRegStr HKCU "Software\Microsoft\Windows\Shell\Associations\UrlAssociations\https\UserChoice" "Progid"  "IE.HTTPS"
  WriteRegStr HKCU "Software\Microsoft\Windows\CurrentVersion\Explorer\FileExts\.html\UserChoice" "ProgId" "IE.AssocFile.HTM"
  WriteRegStr HKCU "Software\Microsoft\Windows\CurrentVersion\Explorer\FileExts\.htm\UserChoice" "ProgId" "IE.AssocFile.HTM"
FunctionEnd

; Don't setup the survey controls, functions, etc. when the application has
; defined NO_UNINSTALL_SURVEY
!ifndef NO_UNINSTALL_SURVEY
Function un.Survey
  Exec "$\"$TmpVal$\" $\"${SurveyURL}$\""
FunctionEnd
!endif

################################################################################
# Language

!insertmacro MOZ_MUI_LANGUAGE 'baseLocale'
!verbose push
!verbose 3
!include "overrideLocale.nsh"
!include "customLocale.nsh"
!verbose pop

; Set this after the locale files to override it if it is in the locale. Using
; " " for BrandingText will hide the "Nullsoft Install System..." branding.
BrandingText " "

################################################################################
# Page pre, show, and leave functions

Function un.preWelcome
  ${If} ${FileExists} "$INSTDIR\distribution\modern-wizard.bmp"
    Delete "$PLUGINSDIR\modern-wizard.bmp"
    CopyFiles /SILENT "$INSTDIR\distribution\modern-wizard.bmp" "$PLUGINSDIR\modern-wizard.bmp"
  ${EndIf}
FunctionEnd

Function un.leaveWelcome
  ${If} ${FileExists} "$INSTDIR\${FileMainEXE}"
    Banner::show /NOUNLOAD "$(BANNER_CHECK_EXISTING)"

    ${If} "$TmpVal" == "FoundMessageWindow"
      Sleep 5000
    ${EndIf}

    ${PushFilesToCheck}

    ${un.CheckForFilesInUse} $TmpVal

    Banner::destroy

    ${If} "$TmpVal" == "true"
      StrCpy $TmpVal "FoundMessageWindow"
      ${un.ManualCloseAppPrompt} "${WindowClass}" "$(WARN_MANUALLY_CLOSE_APP_UNINSTALL)"
      StrCpy $TmpVal "true"
    ${EndIf}
  ${EndIf}
FunctionEnd

Function un.preConfirm
  ${If} ${FileExists} "$INSTDIR\distribution\modern-header.bmp"
  ${AndIf} $hHeaderBitmap == ""
    Delete "$PLUGINSDIR\modern-header.bmp"
    CopyFiles /SILENT "$INSTDIR\distribution\modern-header.bmp" "$PLUGINSDIR\modern-header.bmp"
    ${un.ChangeMUIHeaderImage} "$PLUGINSDIR\modern-header.bmp"
  ${EndIf}

  !insertmacro un.createUnConfirmINI
  !insertmacro MUI_HEADER_TEXT "$(UN_CONFIRM_PAGE_TITLE)" "$(UN_CONFIRM_PAGE_SUBTITLE)"
  ; The Summary custom page has a textbox that will automatically receive
  ; focus. This sets the focus to the Install button instead.
  !insertmacro MUI_INSTALLOPTIONS_INITDIALOG "unconfirm.ini"
  GetDlgItem $0 $HWNDPARENT 1
  ${MUI_INSTALLOPTIONS_READ} $1 "unconfirm.ini" "Field 4" "HWND"
  SetCtlColors $1 0x000000 0xFFFFEE
  ShowWindow $1 ${SW_HIDE}
  System::Call "user32::SetFocus(i r0, i 0x0007, i,i)i"
  ${MUI_INSTALLOPTIONS_READ} $1 "unconfirm.ini" "Field 2" "HWND"
  SendMessage $1 ${WM_SETTEXT} 0 "STR:$INSTDIR"
  !insertmacro MUI_INSTALLOPTIONS_SHOW
FunctionEnd

Function un.leaveConfirm
  ${MUI_INSTALLOPTIONS_READ} $0 "unconfirm.ini" "Settings" "State"
  StrCmp $0 "3" +1 continue
  ${MUI_INSTALLOPTIONS_READ} $0 "unconfirm.ini" "Field 3" "State"
  ${MUI_INSTALLOPTIONS_READ} $1 "unconfirm.ini" "Field 4" "HWND"
  StrCmp $0 1 +1 +3
  ShowWindow $1 ${SW_SHOW}
  Abort

  ShowWindow $1 ${SW_HIDE}
  Abort

  continue:

  ; Try to delete the app executable and if we can't delete it try to find the
  ; app's message window and prompt the user to close the app. This allows
  ; running an instance that is located in another directory. If for whatever
  ; reason there is no message window we will just rename the app's files and
  ; then remove them on restart if they are in use.
  ClearErrors
  ${DeleteFile} "$INSTDIR\${FileMainEXE}"
  ${If} ${Errors}
    ${un.ManualCloseAppPrompt} "${WindowClass}" "$(WARN_MANUALLY_CLOSE_APP_UNINSTALL)"
  ${EndIf}
FunctionEnd

!ifndef NO_UNINSTALL_SURVEY
Function un.preFinish
  ; Do not modify the finish page if there is a reboot pending
  ${Unless} ${RebootFlag}
    ; Setup the survey controls, functions, etc.
    StrCpy $TmpVal "SOFTWARE\Microsoft\IE Setup\Setup"
    ClearErrors
    ReadRegStr $0 HKLM $TmpVal "Path"
    ${If} ${Errors}
      ;!insertmacro MUI_INSTALLOPTIONS_WRITE "ioSpecial.ini" "settings" "NumFields" "3"
      ClearErrors
      ReadRegStr $TmpVal HKLM "SOFTWARE\Clients\StartMenuInternet\IEXPLORE.EXE\shell\open\command"  ""
    ${Else}
      ExpandEnvStrings $0 "$0" ; this value will usually contain %programfiles%
      ${If} $0 != "\"
        StrCpy $0 "$0\"
      ${EndIf}
      StrCpy $0 "$0\iexplore.exe"
      ClearErrors
      GetFullPathName $TmpVal $0
      ${If} ${Errors}
        !insertmacro MUI_INSTALLOPTIONS_WRITE "ioSpecial.ini" "settings" "NumFields" "3"
      ${Else}
        ; When we add an optional action to the finish page the cancel button
        ; is enabled. This disables it and leaves the finish button as the
        ; only choice.
        !insertmacro MUI_INSTALLOPTIONS_WRITE "ioSpecial.ini" "settings" "cancelenabled" "0"
      ${EndIf}
    ${EndIf}
  ${EndUnless}
FunctionEnd
!endif

################################################################################
# Initialization Functions

Function .onInit
  ${UninstallOnInitCommon}
FunctionEnd

Function un.onInit
  StrCpy $STATUS ""
  StrCpy $StartTime ""
  time::_GetLocalTime /NOUNLOAD 
  Pop $StartTime 
  ReadRegStr $0 HKLM "Software\Clients\StartMenuInternet\FLOCK.EXE\shell\open\command" ""
  IfErrors noflock 0
    Push $0 # contains the path to flock.exe
    ${un.GetParent} "$0" $0
    Pop $0 # Contains the flock install folder
    IfFileExists "$0\defaults\pref\uuid.js" 0 skipuuid
      FileOpen $9 $0\defaults\pref\uuid.js r
      FileRead $9 $1
      FileClose $9 
      StrCpy $GUID $1 -3 28
   skipuuid:
   # Use the existing $GUID if we can't find one on the FS
  noflock:
  ClearErrors
  Call un.VistaCheck
  proc_check:
  sleep 500
  Processes::FindProcess "flock.exe"
  ${If} $R0 != 0
    MessageBox MB_OKCANCEL "$(CurrentlyRunningUninstall)" IDOK proc_check IDCANCEL 0
    Abort "$(UninstallCancel)"
  ${EndIf}
  GetFullPathName $INSTDIR "$INSTDIR\.."
  ${un.GetLongPath} "$INSTDIR" $INSTDIR
  ${Unless} ${FileExists} "$INSTDIR\${FileMainEXE}"
    Abort
  ${EndUnless}

  StrCpy $LANGUAGE 0
  ${un.SetBrandNameVars} "$INSTDIR\distribution\setup.ini"

  ; Initialize $hHeaderBitmap to prevent redundant changing of the bitmap if
  ; the user clicks the back button
  StrCpy $hHeaderBitmap ""
FunctionEnd

Function .onGUIEnd
  ${OnEndCommon}
FunctionEnd

Function un.onGUIEnd
  ${un.OnEndCommon}
FunctionEnd

Function un.VistaCheck

  ${If} ${AtLeastWinVista}
    ;MessageBox MB_OK "Debug : Vista"
    UAC::IsAdmin
    ; If the user is not an admin already
    ${If} "$0" != "1"
      ;MessageBox MB_OK "Debug: Not Admin : $0"
      UAC::SupportsUAC
      ;MessageBox MB_OK "Debug: Supports UAC : $0"
      ; If the system supports UAC ( in $0, should be 1)
      UAC::GetElevationType
      ;MessageBox MB_OK "Debug: Current Elevation : $0"
      UAC::RunElevated
      ;MessageBox MB_OK "Debug: Elevation Result : $0"
      UAC::Unload
      Quit
    ${Else}
      UAC::RunElevated 
    ${EndIf}
  ${EndIf}
  
FunctionEnd

; How long since the installer started running?
; Pop $VAR to find out
Function HowLong
  time::_GetLocalTime 
  Pop $R0 
  time::_MathTime /NOUNLOAD "second($R0) - second($StartTime) = second"
FunctionEnd

Function CallmeBack
  ; Send the instrumentation data back to the server.
  Call HowLong
  Pop $R1
  inetc::get /CAPTION " " /USERAGENT "FlockUNINSTALL" /BANNER "Please Wait.." /NOCANCEL /END "http://metrics.flock.com/flock-uninstaller?V${AppVersion}&$STATUS&$R1&$Metrics&GUID=$GUID" "$PLUGINSDIR\temp"
  ;inetc::get /CAPTION " " /USERAGENT "FlockINSTALL" /BANNER "Please Wait.." /NOCANCEL /END "http://greenbtn.com/flock-installer?V${AppVersion}&$STATUS&TIME$R1$Metrics" "$PLUGINSDIR\temp"
  Pop $0
  time::_unload
  ;MessageBox MB_OK "EXIT CODE: [$STATUS] TIME IN INSTALLER: [$R1] METRICS DATA: [$METRICS]"
FunctionEnd

Function BuildString
  Call HowLong
  Pop $0
  FunctionEnd

Function un.HowLong
  time::_GetLocalTime 
  Pop $R0 
  time::_MathTime /NOUNLOAD "second($R0) - second($StartTime) = second"
FunctionEnd

Function un.CallmeBack
  ; Send the instrumentation data back to the server.
  Call un.HowLong
  Pop $R1
  inetc::get /CAPTION " " /USERAGENT "FlockUNINSTALL" /BANNER "Please Wait.." /NOCANCEL /END "http://metrics.flock.com/flock-uninstaller?V${AppVersion}&$STATUS&$R1&$Metrics" "$PLUGINSDIR\temp"
  ;inetc::get /CAPTION " " /USERAGENT "FlockINSTALL" /BANNER "Please Wait.." /NOCANCEL /END "http://greenbtn.com/flock-installer?V${AppVersion}&$STATUS&TIME$R1$Metrics" "$PLUGINSDIR\temp"
  Pop $0
  time::_unload
  ;MessageBox MB_OK "EXIT CODE: [$STATUS] TIME IN INSTALLER: [$R1] METRICS DATA: [$METRICS]"
FunctionEnd


Function un.BuildString
  Call un.HowLong
  Pop $0
FunctionEnd

Function un.onInstFailed
  StrCpy $Status "FAILED"
  Call un.BuildString
  Call un.CallmeBack
  UAC::Unload ;Must call unload!
FunctionEnd

Function un.onInstSuccess
  StrCpy $Status "SUCCESS"
  Call un.BuildString
  Call un.CallmeBack
  UAC::Unload ;Must call unload!
FunctionEnd

Function Custom.onUserAbort
  StrCpy $Status "ABORT"
  Call BuildString
  Call CallmeBack
  UAC::Unload ;Must call unload!
FunctionEnd
