/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: se cin sw=2 ts=2 et : */
#ifndef nsDownloadScanner_h_
#define nsDownloadScanner_h_

#ifdef WIN32_LEAN_AND_MEAN
#undef WIN32_LEAN_AND_MEAN
#endif
#define INITGUID
#include <Windows.h>

#if _MSC_VER < 1400
// IAttachmentExecute interface
// NOTE: This interface requires MS Visual Studio 2005 or later to compile.
// Imported from SHDOCVW.DLL/SHDOCVW.LIB:
//EXTERN_C const IID      IID_IAttachmentExecute;
//EXTERN_C const CLSID    CLSID_AttachmentServices;
// XXX These are TOTALLY BOGUS substitutes so I can finish compiling:
const IID IID_IAttachmentExecute = {0xFFFFFFFF,0x803A,0x11D0,{0xAC,0x63,0x00,0xC0,0x4F,0xD9,0x75,0x75}};
const CLSID CLSID_AttachmentServices = {0xFFFFFFFF,0x5C6E,0x11D1,{0x9E,0xC1,0x00,0xC0,0x4F,0xD7,0x08,0x1F}};

typedef enum tagATTACHMENT_PROMPT {
    ATTACHMENT_PROMPT_NONE          = 0x00000000,
    ATTACHMENT_PROMPT_SAVE          = 0x00000001,
    ATTACHMENT_PROMPT_EXEC          = 0x00000002,
    ATTACHMENT_PROMPT_EXEC_OR_SAVE  = 0x00000003
} ATTACHMENT_PROMPT;

typedef enum tagATTACHMENT_ACTION {
    ATTACHMENT_ACTION_CANCEL    = 0x00000000,
    ATTACHMENT_ACTION_SAVE      = 0x00000001,
    ATTACHMENT_ACTION_EXEC      = 0x00000002
} ATTACHMENT_ACTION;

#define INTERFACE IAttachmentExecute
DECLARE_INTERFACE_( IAttachmentExecute, IUnknown ) {
    /* IUnknown methods */
    STDMETHOD( QueryInterface )( THIS_ REFIID, void ** ) PURE;
    STDMETHOD_( ULONG, AddRef )( THIS ) PURE;
    STDMETHOD_( ULONG, Release )( THIS ) PURE;

    /* IAttachmentExecute methods */
    STDMETHOD( SetClientTitle )( THIS_ LPCWSTR ) PURE;
    STDMETHOD( SetClientGuid )( THIS_ REFGUID ) PURE;
    STDMETHOD( SetLocalPath )( THIS_ LPCWSTR ) PURE;
    STDMETHOD( SetFileName )( THIS_ LPCWSTR ) PURE;
    STDMETHOD( SetSource )( THIS_ LPCWSTR ) PURE;
    STDMETHOD( SetReferrer )( THIS_ LPCWSTR ) PURE;
    STDMETHOD( CheckPolicy )( THIS ) PURE;
    STDMETHOD( Prompt )( THIS_ HWND, ATTACHMENT_PROMPT, ATTACHMENT_ACTION * ) PURE;
    STDMETHOD( Save )( THIS ) PURE;
    STDMETHOD( Execute )( THIS_ HWND, LPCWSTR, HANDLE * ) PURE;
    STDMETHOD( SaveWithUI )( THIS_ HWND ) PURE;
    STDMETHOD( ClearClientState )( THIS ) PURE;
};
#undef INTERFACE
// END IAttachmentExecute interface
#endif /* _MSC_VER < 1400 */

#define AVVENDOR
#include <msoav.h>
// To cope with both msvs8 header and sdk6 header
#ifdef _WIN32_IE_IE60SP2
#undef _WIN32_IE
#define _WIN32_IE _WIN32_IE_IE60SP2
#endif
#include <shlobj.h>

#include "nsAutoPtr.h"
#include "nsThreadUtils.h"
#include "nsTArray.h"
#include "nsIObserver.h"
#include "nsIURI.h"

enum AVScanState
{
  AVSCAN_NOTSTARTED = 0,
  AVSCAN_SCANNING,
  AVSCAN_GOOD,
  AVSCAN_BAD,
  AVSCAN_UGLY,
  AVSCAN_FAILED,
  AVSCAN_TIMEDOUT
};

enum AVCheckPolicyState
{
  AVPOLICY_DOWNLOAD,
  AVPOLICY_PROMPT,
  AVPOLICY_BLOCKED
};

// See nsDownloadScanner.cpp for declaration and definition
class nsDownloadScannerWatchdog;
class nsDownload;

class nsDownloadScanner : public nsIObserver
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIOBSERVER

public:
  nsDownloadScanner();
  ~nsDownloadScanner();
  nsresult Init();
  nsresult ScanDownload(nsDownload *download);
  AVCheckPolicyState CheckPolicy(nsIURI *aSource, nsIURI *aTarget);

private:
  PRBool mOAVExists;
  PRBool mAESExists;
  PRBool mUseAttachmentExecute;
  nsTArray<CLSID> mScanCLSID;
  PRBool IsAESAvailable();
  PRBool EnumerateOAVProviders();

  nsAutoPtr<nsDownloadScannerWatchdog> mWatchdog;

  static unsigned int __stdcall ScannerThreadFunction(void *p);
  class Scan : public nsRunnable
  {
  public:
    Scan(nsDownloadScanner *scanner, nsDownload *download);
    ~Scan();
    nsresult Start();

    // Returns the time that Start was called
    PRTime GetStartTime() const { return mStartTime; }
    // Returns a copy of the thread handle that can be waited on, but not
    // terminated
    // The caller is responsible for closing the handle
    // If the thread has terminated, then this will return the pseudo-handle
    // INVALID_HANDLE_VALUE
    HANDLE GetWaitableThreadHandle() const;

    // Called on a secondary thread to notify the scan that it has timed out
    // this is used only by the watchdog thread
    PRBool NotifyTimeout();

  private:
    nsDownloadScanner *mDLScanner;
    PRTime mStartTime;
    HANDLE mThread;
    nsRefPtr<nsDownload> mDownload;
    // Guards mStatus
    CRITICAL_SECTION mStateSync;
    AVScanState mStatus;
    nsString mPath;
    nsString mName;
    nsString mOrigin;
    // Also true if it is an ftp download
    PRBool mIsHttpDownload;
    PRBool mSkipSource;
    PRBool mIsReadOnlyRequest;

    /* @summary Sets the Scan's state to newState if the current state is
                expectedState
     * @param newState The new state of the scan
     * @param expectedState The state that the caller expects the scan to be in
     * @return If the old state matched expectedState
     */
    PRBool CheckAndSetState(AVScanState newState, AVScanState expectedState);

    NS_IMETHOD Run();

    void DoScan();
    PRBool DoScanAES();
    PRBool DoScanOAV();

    friend unsigned int __stdcall nsDownloadScanner::ScannerThreadFunction(void *);
  };
  // Used to give access to Scan
  friend class nsDownloadScannerWatchdog;
};
#endif
